package	jp.co.sds_corp.util.common;

import java.io.Serializable;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.TimeZone;

/**
* 日付の操作を行ないます
* @author	小見山　聡
*/
public class CmnDate extends GregorianCalendar
	implements Comparable<Calendar>, Cloneable, Serializable
{
	private static final long serialVersionUID = 1L;

	/**
	* コンストラクタ
	* 現在時刻で初期化されたCmnDateオブジェクトを作成します。
	*/
	public CmnDate() {
		super();
	}
	/**
	* コンストラクタ
	* 指定された日付(YYYYMMDD)で初期化されたCmnDateオブジェクトを作成します。
	* @param	d		設定する日付
	*/
	public CmnDate( String d ) throws ParseException {
		setDate( d );
	}
	/**
	* コンストラクタ
	* 指定された日付を指定されたフォーマット文字列でで初期化された
	* CmnDateオブジェクトを作成します。
	* @param	d		設定する日付
	* @param	fmt		フォーマット文字列
	*/
	public CmnDate( String d, String fmt ) throws ParseException {
		setDate( d, fmt );
	}
	/**
	* コンストラクタ
	* 指定された日付を指定されたフォーマット文字列でで初期化された
	* CmnDateオブジェクトを作成します。
	* @param	d		設定する日付
	* @param	fmt		フォーマット文字列
	* @param	loc		Locale
	*/
	public CmnDate( String d, String fmt, Locale loc ) throws ParseException {
		setDate( d, fmt, loc );
	}
	/**
	* コンストラクタ
	* 指定された日付で初期化されたCmnDateオブジェクトを作成します。
	* @param	d		設定する日付
	*/
	public CmnDate( Date d ) {
		setDate( d );
	}

	/**
	* 指定された日付(YYYYMMDD)でCmnDateオブジェクトを初期化します。
	* @param	d		設定する日付
	*/
	public void setDate( String d ) throws ParseException {
		SimpleDateFormat sdp = new SimpleDateFormat( "yyyyMMdd" );
		Date dt = sdp.parse( d );
		setTime( dt );
	}
	/**
	* 指定された日付と指定されたフォーマット文字列で
	* CmnDateオブジェクトを初期化します。
	* @param	d		設定する日付
	* @param	fmt		フォーマット文字列
	*/
	public void setDate( String d, String fmt ) throws ParseException {
		setDate( d, fmt, CmnLocale.getLocale() );
	}
	/**
	* 指定された日付と指定されたフォーマット文字列で
	* CmnDateオブジェクトを初期化します。
	* @param	d		設定する日付
	* @param	fmt		フォーマット文字列
	* @param	loc		Locale
	*/
	public void setDate( String d, String fmt, Locale loc ) throws ParseException {
		SimpleDateFormat sdp = new SimpleDateFormat( fmt, loc );
		sdp.setLenient( false );
		Date dt = sdp.parse( d );
		setTime( dt );
	}
	/**
	* 指定された日付でCmnDateオブジェクトを初期化します。
	* @param	d		設定する日付
	*/
	public void setDate( Date d ) {
		setTime( d );
	}

	/**
	* 日付の時分秒を0に設定します。
	*/
	public void clearTime() {
		if ( isSet( AM_PM )) {
			set( AM_PM, AM );
		}
		if ( isSet( HOUR )) {
			clear( HOUR );
		}
		if ( isSet( MINUTE )) {
			clear( MINUTE );
		}
		if ( isSet( SECOND )) {
			clear( SECOND );
		}
		if ( isSet( MILLISECOND )) {
			clear( MILLISECOND );
		}
		if ( isSet( HOUR_OF_DAY )) {
			clear( HOUR_OF_DAY );
		}
	}

	/**
	* 日付を日付／時刻文字列にフォーマットします。
	* @param	fmt		フォーマット文字列
	* @param	loc		Locale
	* @param	tz		タイムゾーン
	* @return	フォーマットされた日時の文字列
	*/
	public String format( String fmt, Locale loc, TimeZone tz ) {
		WarekiFormat df = new WarekiFormat( fmt, loc, tz );
		Date dt = getTime();
		return df.format( dt );
	}

	/**
	* 日付を日付／時刻文字列にフォーマットします。
	* @param	fmt		フォーマット文字列
	* @param	tz		タイムゾーン
	* @return	フォーマットされた日時の文字列
	*/
	public String format( String fmt, TimeZone tz ) {
		return format( fmt, Locale.getDefault(), tz );
	}

	/**
	* 日付を日付／時刻文字列にフォーマットします。
	* @param	fmt		フォーマット文字列
	* @param	loc		Locale
	* @return	フォーマットされた日時の文字列
	*/
	public String format( String fmt, Locale loc ) {
		return format( fmt, loc, TimeZone.getDefault() );
	}

	/**
	* 日付を日付／時刻文字列にフォーマットします。
	* 和暦変換も行います。
	* @param	fmt		フォーマット文字列
	* @return	フォーマットされた日時の文字列
	*/
	public String format( String fmt ) {
		return format( fmt, Locale.getDefault(), TimeZone.getDefault() );
	}

	/**
	* 日付をyyyyMMdd文字列にフォーマットします。
	* @return	フォーマットされた日時の文字列
	*/
	public String getString() {
		SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );
		Date dt = getTime();
		return sdf.format( dt );
	}

	/**
	* 2つの日付の差を計算します。(片端) this - dt
	* @param	dt		引く日
	* @return	差の日数
	*/
	public long diffDays( CmnDate dt ) {
		CmnDate dt1, dt2;
		dt1 = (CmnDate )clone();
		dt2 = (CmnDate )dt.clone();
		dt1.clearTime();
		dt2.clearTime();
		return ((dt1.getTimeInMillis() - dt2.getTimeInMillis())
					/ (24*60*60) / 1000 );
	}

	/**
	* 2つの日付を比較します。
	* @param	obj		比較する日付
	* @return	引数がこの日付と等しい日付の場合は値 0。
	*			引数がこの日付より後の日付場合は 0 より小さい値。
	*			引数がこの日付の前の日付の場合は 0 より大きい値。
	*/
	public int compareTo( CmnDate obj ) {
		long lDiff = getTimeInMillis() - ((CmnDate )obj).getTimeInMillis();
		int nRet = ( lDiff == 0 ) ? 0 : ((lDiff > 0 ) ? 1 : -1 );
		return nRet;
//		return (int )(getTimeInMillis())
//			- (int )(((CmnDate )obj).getTimeInMillis());
//		return (int )diffDays( (CmnDate )obj );
	}

	/**
	* 現在時刻を取得します。
	* @return	エポックからの UTC ミリ秒値で表される現在時刻
	*/
	public long getTimeInMillis() {
		return super.getTimeInMillis();
	}

	/**
	* 現在時刻をjava.sql.Timestamp形式で取得します。
	* @return	現在時刻
	*/
	public java.sql.Timestamp getTimestamp() {
		return new java.sql.Timestamp( getTime().getTime() );
	}

	/**
	* 日付がヌル値(ブランク)かどうかを判定する
	* @return	true:ヌルである　false:ヌルではない
	*/
	public boolean isNull() {
		boolean ret = false;
		try {
			CmnDate dt = new CmnDate( "19000101" );
			ret = equals( dt );
		} catch ( Exception ex ) {
		}
		return ret;
	}
}
