package	jp.co.sds_corp.util.common;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
* プロパティファイルの操作用クラスです。
* @author	小見山　聡
*/
public class CmnProperty
{
	ResourceBundle infoProp;
	String	baseName;
	Locale	loc;

	/**
	* プロパティファイルを指定された基底名、デフォルトのロケールで取得します。
	* @param	baseName	基底名
	*/
	public CmnProperty( String baseName ) {
		this.baseName = baseName;
		this.loc = CmnLocale.getLocale();
		createBundle();
	}

	/**
	* プロパティファイルを指定された基底名、ロケールで取得します。
	* @param	baseName	基底名
	* @param	loc			ロケール
	*/
	public CmnProperty( String baseName, Locale loc ) {
		this.baseName = baseName;
		this.loc = loc;
		createBundle();
	}

	/**
	* プロパティファイルを、ファイルから直接取得します。
	* @param	dir			ディレクトリ
	* @param	baseName	ファイル名
	*/
	public CmnProperty( String dir, String baseName )
		throws FileNotFoundException, IOException
	{
		String file = dir + File.separator + baseName + ".properties";
		FileInputStream is = new FileInputStream( file );
		infoProp = new PropertyResourceBundle( is );
	}

	/**
	* プロパティファイルを、ファイルから直接取得します。
	* @param	file	ファイル
	*/
	public CmnProperty( File file )
		throws FileNotFoundException, IOException
	{
		FileInputStream is = new FileInputStream( file );
		infoProp = new PropertyResourceBundle( is );
	}

	protected void createBundle() {
		infoProp = PropertyResourceBundle.getBundle( baseName, loc );
	}

	/**
	* キーの一覧を取得します。
	* @return	キーの一覧
	*/
	public Enumeration<String> getKeys() {
		return infoProp.getKeys();
	}
	/**
	* プロパティファイルのロケールを取得します。
	* @return	プロパティファイルのロケール
	*/
	public Locale getLocale() {
		return infoProp.getLocale();
	}

	/**
	* プロパティファイルから指定されたキーのオブジェクトを取得します。
	* @param	key		キー
	*/
	public Object getObject( String key ) {
		try {
			return infoProp.getObject( key );
		} catch ( java.util.MissingResourceException ex ) {
			return null;
		}
	}

	Pattern ptn = Pattern.compile( "\\$\\{([\\w.]+)\\}" );
	/**
	* 文字列をプロパティファイルの内容で置換します。
	* ${}で囲まれた文字列があった場合、カッコ内の文字列をキーとして
	* プロパティファイルを検索し、その値を埋め込みます。
	* 「${dir}/logs/」を、「/home/XXX/logs」という文字列に変換します。
	* プロパティファイルにキーが見つからない場合には、システムプロパティを
	* 検索します。
	* @param	buf		置換される文字列
	* @return	変換後の文字列
	*/
	public String replaceValue( String buf ) {
		Matcher matcher = ptn.matcher( buf );
		if ( !matcher.find()) {
			return buf;
		}
		String key = matcher.group( 1 );
		String var = getString( key );
		if ( var == null ) {
			var = System.getProperty( key );
			if ( var == null ) {
				return buf;
			}
		}
		return matcher.replaceFirst( var );
	}

	/**
	* プロパティファイルから指定されたキーの文字列を取得します。
	* ${}で囲まれた文字列があった場合、{@link #replaceValue(String)}を使用して
	* プロパティの値に変換します。
	* <p>
	* 例)
	* dir=/home/XXX
	* logdir=${dir}/logs
	* の場合、getString( "logdir" )は、/home/XXX/logsを返します。
	* </p>
	* @param	key		キー
	* @return	指定されたキーの文字列
	* @see	#replaceValue
	*/
	public String getString( String key ) throws MissingResourceException {
		String ret = null;
		ret = infoProp.getString( key );
		for ( ;; ) {
			String rep = replaceValue( ret );
			if ( rep.equals( ret )) {
				break;
			}
			ret = rep;
		}
		return ret;
	}

	/**
	* プロパティファイルから指定されたキーの文字列を取得します。
	* 指定された値が見つからなかった場合は、指定されたデフォルト値を返します。
	* 文字列の変換は、{@link #getString(String)}と同じです。
	* @param	key		キー
	* @param	defVal	デフォルト値
	* @return	指定されたキーの文字列。見つからなかった場合はデフォルト値
	*/
	public String getString( String key, String defVal ) {
		String ret = null;
		try {
			ret = getString( key );
		} catch ( MissingResourceException ex ) {
			ret = defVal;
		}
		return ret;
	}

	/**
	* プロパティファイルから指定されたキーの文字列の配列を取得します。
	* @param	key		キー
	* @return	指定されたキーの文字列
	*/
	public String[] getStringArray( String key ) {
		String[] arr = null;
		try {
			arr = infoProp.getStringArray( key );
			for ( int i = 0; i < arr.length; i++ ) {
				for ( ;; ) {
					String rep = replaceValue( arr[i] );
					if ( rep.equals( arr[i] )) {
						break;
					}
					arr[i] = rep;
				}
			}
		} catch ( java.util.MissingResourceException ex ) {
			return null;
		}
		return arr;
	}

	/**
	* プロパティファイルから指定されたキーの文字列の配列を取得します。
	* 取得した文字列を、デリミタで分割して配列にして返します。
	* @param	key		キー
	* @param	delim	デリミタ
	* @return	指定されたキーの文字列
	*/
	public String[] getStringToken( String key, String delim ) {
		String val = null;
		try {
			val = infoProp.getString( key );
		} catch ( java.util.MissingResourceException ex ) {
			return null;
		}
		StringTokenizer tok;
		if ( delim == null ) {
			tok = new StringTokenizer( val );
		} else {
			tok = new StringTokenizer( val, delim );
		}
		int nNum = tok.countTokens();
		String[] res = new String[nNum];
		for ( int i = 0; i < nNum; i++ ) {
			if ( delim == null ) {
				res[i] = tok.nextToken();
			} else {
				res[i] = tok.nextToken( delim );
			}
		}
		return res;
	}
}

