package jp.co.sds_corp.util.crypt;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.security.Key;
import java.security.KeyFactory;
import java.security.interfaces.RSAKey;
import java.security.spec.KeySpec;

import javax.crypto.Cipher;

import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public abstract class AbstractCipher implements ICipher, IConfigurable
{
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	String	algorithm = "RSA";
	String	keyAlgorithm = "RSA";
	File	file;
	InputStream	is;
	String	password;
	boolean	privateKey;

	@Override
	public void init(IConfiguration config) throws Exception {
		IConfiguration attr = config.getAttributes();
		String filename = attr.get("file", null);
		if (filename != null && filename.length() > 0) {
			setFile(new File(filename));
		}
		String algorithm = attr.get("algorithm", null);
		if (algorithm != null && algorithm.length() > 0) {
			setAlgorithm(algorithm);
		}
		String keyAlgorithm = attr.get("keyAlgorithm", null);
		if (keyAlgorithm != null && keyAlgorithm.length() > 0) {
			setKeyAlgorithm(keyAlgorithm);
		}

		if (this instanceof NeedPassword) {
			String password = attr.get("password", null);
			if (password != null && password.length() > 0) {
				setPassword(password);
			}
		}
	}

	@Override
	public byte[] encrypt(byte[] plain) throws Exception {
		Key key = getKey();
		logger.trace("encrypt key=" + key);

		logger.trace("encrypt algorithm=" + getAlgorithm());
		Cipher cipher = Cipher.getInstance(getAlgorithm());
		cipher.init(Cipher.ENCRYPT_MODE, key);

		int blockSize = getEncryptBlockSize(key);
		BlockCipher block = new BlockCipher(cipher, blockSize);
		byte[] encrypted = block.doFinal(plain);
		return encrypted;
	}

	@Override
	public byte[] decrypt(byte[] encrypted) throws Exception {
		Key key = getKey();
		logger.trace("decrypt key=" + key);

		logger.trace("decrypt algorithm=" + getAlgorithm());
		Cipher cipher = Cipher.getInstance(getAlgorithm());
		cipher.init(Cipher.DECRYPT_MODE, key);

		int blockSize = getDecryptBlockSize(key);
		BlockCipher block = new BlockCipher(cipher, blockSize);
		byte[] decrypted = block.doFinal(encrypted);
		return decrypted;
	}

	public Key getKey() throws Exception {
		KeyFactory keyFactory = KeyFactory.getInstance(getKeyAlgorithm());
		KeySpec spec = getKeySpec();
		logger.trace("keyspec=" + spec);
		Key key;
		if (isPrivateKey()) {
			key = keyFactory.generatePrivate(spec);
		} else {
			key = keyFactory.generatePublic(spec);
		}
		return key;
	}

	public abstract KeySpec getKeySpec() throws Exception;

	protected byte[] load() throws Exception {
		byte[] ret = null;
		if (getFile() != null) {
			ret = load(getFile());
		} else
		if (getInputStream() != null) {
			ret = load(getInputStream());
		}
		return ret;
	}

	protected byte[] load(File file) throws Exception {
		byte[] ret = null;
		FileInputStream fis = null;
		try {
			fis = new FileInputStream(file);
			ret = load(fis);
		} finally {
			if (fis != null) {
				fis.close();
			}
		}
		return ret;
	}

	protected byte[] load(InputStream is) throws Exception {
		byte[] buf = new byte[is.available()];
		is.read(buf);
		return buf;
	}

	@Override
	public void setAlgorithm(String algorithm) {
		this.algorithm = algorithm;
	}

	public String getAlgorithm() {
		return this.algorithm;
	}

	@Override
	public void setKeyAlgorithm(String algorithm) {
		this.keyAlgorithm = algorithm;
	}

	public String getKeyAlgorithm() {
		return this.keyAlgorithm;
	}

	@Override
	public void setFile(File file) {
		this.file = file;
	}

	public File getFile() {
		return this.file;
	}

	@Override
	public void setInputStream(InputStream is) {
		this.is = is;
	}

	public InputStream getInputStream() {
		return this.is;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getPassword() {
		return this.password;
	}

	public void setPrivateKey(boolean privateKey) {
		this.privateKey = privateKey;
	}

	public boolean isPrivateKey() {
		return this.privateKey;
	}

	public int getEncryptBlockSize(Key key) {
		int ret = -1;
		if (key instanceof RSAKey) {
			ret = ((RSAKey )key).getModulus().bitLength() / 8 - 11;
		}
		return ret;
	}
	public int getDecryptBlockSize(Key key) {
		int ret = -1;
		if (key instanceof RSAKey) {
			ret = ((RSAKey )key).getModulus().bitLength() / 8;
		}
		return ret;
	}

}

