package jp.co.sds_corp.util.diag;

import java.lang.management.LockInfo;
import java.lang.management.ManagementFactory;
import java.lang.management.MonitorInfo;
import java.lang.management.ThreadInfo;
import java.lang.management.ThreadMXBean;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.List;

import jp.co.sds_corp.util.log.ConsoleLogger;

public class ThreadMonitor
{
	ConsoleLogger logger = ConsoleLogger.getLogger(this);
	boolean showStack = false;
	Map<Long,ThreadInfo> mapInfo = new HashMap<Long,ThreadInfo>();

	public static ThreadGroup getTopThreadGroup() {
		ThreadGroup top = null;
		ThreadGroup parent;

		top = Thread.currentThread().getThreadGroup();
		for ( ;; ) {
			parent = top.getParent();
			if (parent == null) {
				break;
			}
			top = parent;
		}
		return top;
	}

	public void displayThreadList() {
		List<String> list = listThread();
		for (String msg : list) {
			logger.info(msg);
		}
	}
	public List<String> listThread() {
		ArrayList<String> ret = new ArrayList<String>();
		ret.add("=== thread list ===");
		ThreadMXBean threadBean = ManagementFactory.getThreadMXBean();
		ThreadInfo[] infos = threadBean.dumpAllThreads(true, true);
		mapInfo.clear();
		for (ThreadInfo info : infos) {
			mapInfo.put(info.getThreadId(), info);
		}
		
		ThreadGroup top = getTopThreadGroup();
		ret.addAll(listThread(top, 0));

		for (ThreadInfo info : mapInfo.values()) {
			ret.addAll(getThreadInfo(info, "* "));
		}
		ret.addAll(getDeadlock());
		return ret;
	}

	List<String> listThread(ThreadGroup group, int indent) {
		ArrayList<String> ret = new ArrayList<String>();
		ret.add(getThreadGroup(group, indent));
		int cnt = group.activeCount();
		if (cnt > 0) {
			Thread[] threads = new Thread[cnt];
			int num = group.enumerate(threads, false);
			for (int i = 0; i < num; i++) {
				long id = threads[i].getId();
				ThreadInfo info = mapInfo.get(id);
				ret.addAll(getThread(threads[i], info, indent + 1));
				mapInfo.remove(id);
			}
		}
		int grpCnt = group.activeGroupCount();
		if (grpCnt > 0) {
			ThreadGroup[] groups = new ThreadGroup[grpCnt];
			int num = group.enumerate(groups, false);
			for (int i = 0; i < num; i++) {
				ret.addAll(listThread(groups[i], indent + 1));
			}
		}
		return ret;
	}

	String getThreadGroup(ThreadGroup group, int indent) {
		String log;
		log = indent(indent) + "Group " + group.getName()
			+ ",ac="+ group.activeCount()
			+ ",agc="+ group.activeGroupCount()
			+ ",pri=" + group.getMaxPriority();
		if (group.isDaemon()) {
			log +=",daemon";
		}
		if (group.isDestroyed()) {
			log +=",destroyed";
		}
		return log;
	}

	List<String> getThread(Thread thread, ThreadInfo info, int indent) {
		ArrayList<String> ret = new ArrayList<String>();
		String log;
		log = indent(indent) + thread.getName()
			+ ",id=" + thread.getId()
			+ ",pri=" + thread.getPriority()
			+ "," + (thread.isAlive() ? "alive" : "dead")
			+ (thread.isDaemon() ? ",daemon" : "")
			+ "," + thread.getState();
		ret.add(log);

		if (showStack) {
			if (info != null) {
				if (info.getLockOwnerId() >= 0) {
					ret.add(indent(indent + 1)
							+ "- owner=" + info.getLockOwnerName()
							+ "[" + info.getLockOwnerId() + "]");
				}
				ret.addAll(getLock(info, indent + 1));

				ret.addAll(getStack(info.getStackTrace(), indent + 1));
			} else {
				ret.addAll(getStack(thread.getStackTrace(), indent + 1));
			}
		}
		return ret;
	}

	List<String> getLock(ThreadInfo info, int indent) {
		ArrayList<String> ret = new ArrayList<String>();
		MonitorInfo[] monitors = info.getLockedMonitors();
		for (MonitorInfo monitor : monitors) {
			ret.add(indent(indent) + "- monitor " + monitor
					+ " at " + monitor.getLockedStackFrame());
		}
		LockInfo lock = info.getLockInfo();
		if (lock != null) {
			ret.add(indent(indent) + "- locked " + lock);
		}
		LockInfo[] syncs = info.getLockedSynchronizers();
		for (LockInfo sync : syncs) {
			ret.add(indent(indent) + "- synchronizer " + sync);
		}
		return ret;
	}

	List<String> getStack(StackTraceElement[] stacks, int indent) {
		ArrayList<String> ret = new ArrayList<String>();
        for ( StackTraceElement stack : stacks ) {
    		String log;
    		log = indent(indent) + stack.toString();
        	ret.add(log);
        }
		return ret;
	}

	List<String> getThreadInfo(ThreadInfo info) {
		return getThreadInfo(info, null);
	}

	List<String> getThreadInfo(ThreadInfo info, String mark) {
		ArrayList<String> ret = new ArrayList<String>();
		ret.add((mark == null ? "" : mark) 
				+ info.getThreadName()
				+ ",id=" + info.getThreadId()
				+ "," + info.getThreadState());

		if (showStack) {
			ret.addAll(getLock(info, 1));
			ret.addAll(getStack(info.getStackTrace(), 1));
		}
		return ret;
	}

	public List<String> getDeadlock() {
		ArrayList<String> ret = new ArrayList<String>();
		ThreadMXBean threadBean = ManagementFactory.getThreadMXBean();
		long[] ids = threadBean.findDeadlockedThreads();
		if (ids == null || ids.length == 0) {
			return ret;
		}

		ret.add("=== deadlock ===");
		for (long id : ids) {
			ThreadInfo info = threadBean.getThreadInfo(id);
			ret.addAll(getThreadInfo(info));
		}
		return ret;
	}

	String indent(int indent) {
		StringBuilder buf = new StringBuilder();
		for (int i = 0; i < indent; i++) {
			buf.append("\t");
		}
		return buf.toString();
	}

	public void setShowStack(boolean var) {
		this.showStack = var;
	}

}
