package	jp.co.sds_corp.util.log;

import java.io.FileNotFoundException;
import java.net.URL;
import java.util.Enumeration;
import java.util.List;
import java.util.Vector;
import java.util.logging.Filter;
import java.util.logging.Formatter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* ログ出力の各クラスをロードし、java.util.logging.LogManagerに設定します。
*/
public class LogLoader
{
	ConsoleLogger console = ConsoleLogger.getLogger(this);
	LogManager		manager = LogManager.getLogManager();
//	ErrorManager	errorManager = new ConsoleErrorManager();
	Vector<Logger>	listLogger = new Vector<Logger>();
	String	root = null;

	public LogLoader() throws Exception {
	}

	/**
	* ログの設定情報を読み込みます。
	*/
	public void init() throws Exception {
		URL config = getClass().getClassLoader().getResource( "logger.xml" );
		if ( config == null ) {
			throw( new FileNotFoundException( "logger.xml" ));
		}
		DOMConfiguration doc = new DOMConfiguration( config );
		init( doc );
	}

	/**
	* ログの設定情報を読み込みます。
	*/
	public void init( IConfiguration doc ) throws Exception {
		List<IConfiguration> list = doc.listChild( "logger" );
		for ( IConfiguration config : list ) {
			Logger logger = null;
			try {
				logger = createLogger( config );
				listLogger.add( logger );
			} catch ( Exception ex ) {
				console.error( "logger load error.", ex );
			}
		}
		// printLoggers();
	}

	/**
	* Loggerを生成し、設定情報で初期化します。
	* @param	config	設定情報
	*/
	protected Logger createLogger( IConfiguration config )
		throws Exception
	{
		String name = config.getAttributes().get( "name" );
		if ( root == null || root.length() > name.length()) {
			root = name;
		}
		boolean useParent = true;
		String var = null;
		try {
		var = config.getAttributes().get( "useParentHandlers" );
		} catch ( Exception ex ) {
		}
		if ( var != null && var.equals( "false" )) {
			useParent = false;
		}
		Logger logger = Logger.getLogger( name );
		if ( logger == null ) {
			console.error( "logger[" + name + "] is null. ignored." );
			return null;
		}
		logger.setUseParentHandlers( useParent );
		clearHandler( logger );

		IConfiguration lcfg = config.getChild( "level" );
		if ( lcfg != null ) {
			Level level = parseLevel( lcfg );
			if ( level != null ) {
				logger.setLevel( level );
			}
		}
		console.debug( "load logger name=" + name
							+ " level=" + logger.getLevel() );

		IConfiguration fcfg = config.getChild( "filter" );
		if ( fcfg != null ) {
			Filter filter = createFilter( fcfg );
			if ( filter != null ) {
				logger.setFilter( filter );
			}
		}

		List<IConfiguration> list = config.listChild( "handler" );
		for ( IConfiguration hcfg : list ) {
			Handler handler = createHandler( hcfg, logger.getLevel() );
			if ( handler != null ) {
				logger.addHandler( handler );
			}
		}
		return logger;
	}

	void clearHandler( Logger logger ) {
		Handler[] handlers = logger.getHandlers();
		for ( Handler handler : handlers ) {
			if ( handler != null ) {
				try {
					handler.close();
				} catch ( Exception ex ) {
				}
				logger.removeHandler( handler );
			}
		}
	}

	protected Level parseLevel( IConfiguration config )
		throws Exception
	{
		Level level = Level.INFO;
		String var = config.getAttributes().get( "value" );
		try {
			level = Level.parse( var );
		} catch ( Exception ex ) {
			console.error( "level parse error.[" + var + "]", ex );
			return null;
		}
		return level;
	}

	protected Object createInstance( String className )
		throws ClassNotFoundException,
				IllegalAccessException, InstantiationException
	{
		Class<?> clazz = Class.forName( className );
		return clazz.newInstance();
	}

/*
SystemClassLoaderからロードするver。
	protected Object createInstance( String className )
		throws ClassNotFoundException,
				IllegalAccessException, InstantiationException
	{
		ClassLoader loader = null;
		try {
			loader = ClassLoader.getSystemClassLoader();
			Class<?> clazz = loader.loadClass( className );
			return clazz.newInstance();
		} catch ( Exception ex ) {
			loader = Thread.currentThread().getContextClassLoader();
			Class<?> clazz = loader.loadClass( className );
			return clazz.newInstance();
		}
	}
*/

	/**
	* Filterを生成し、設定情報で初期化します。
	* @param	config	設定情報
	*/
	protected Filter createFilter( IConfiguration config )
		throws Exception
	{
		Filter filter = null;
		String cls = config.getAttributes().get( "class" );
		console.debug( "load filter " + cls );
		try {
			filter = (Filter )createInstance( cls );
		} catch ( Exception ex ) {
			console.error( "filter load error." + " class=" + cls, ex );
			return null;
		}
		if ( filter instanceof IConfigurable ) {
			((IConfigurable )filter).init( config );
		}
		return filter;
	}

	/**
	* Handlerを生成し、設定情報で初期化します。
	* @param	config	設定情報
	* @param	defaultLevel	デフォルトのログレベル
	*/
	protected Handler createHandler( IConfiguration config, Level defaultLevel )
		throws Exception
	{
		Handler handler = null;
		String cls = config.getAttributes().get( "class" );
		try {
			handler = (Handler )createInstance( cls );
		} catch ( Exception ex ) {
			console.error( "handler load error." + " class=" + cls, ex );
			return null;
		}
		if ( handler instanceof IConfigurable ) {
			((IConfigurable )handler).init( config );
		}

//		handler.setErrorManager( errorManager );
		handler.setLevel( defaultLevel );
		IConfiguration lcfg = config.getChild( "level" );
		if ( lcfg != null ) {
			Level level = parseLevel( lcfg );
			if ( level != null ) {
				handler.setLevel( level );
			}
		}
		console.debug( "load handler " + cls
							+ " level=" + handler.getLevel() );

		IConfiguration fcfg = config.getChild( "filter" );
		if ( fcfg != null ) {
			Filter filter = createFilter( fcfg );
			if ( filter != null ) {
				handler.setFilter( filter );
			}
		}

		IConfiguration fmtcfg = config.getChild( "formatter" );
		Formatter formatter = createFormatter( fmtcfg );
		if ( formatter != null ) {
			handler.setFormatter( formatter );
		}
		return handler;
	}

	/**
	* Formatterを生成し、設定情報で初期化します。
	* @param	config	設定情報
	*/
	protected Formatter createFormatter( IConfiguration config )
		throws Exception
	{
		Formatter formatter = null;
		String cls = config.getAttributes().get( "class" );
		console.debug( "load formatter " + cls );
		try {
			formatter = (Formatter )createInstance( cls );
		} catch ( Exception ex ) {
			console.error( "formatter load error." + " class=" + cls, ex );
		}
		if ( formatter instanceof IConfigurable ) {
			((IConfigurable )formatter).init( config );
		}
		return formatter;
	}

	public String getRoot() {
		return root;
	}

	private void printLoggers() {
		StringBuilder msg = new StringBuilder();
		Enumeration<String> en = manager.getLoggerNames();
		while ( en.hasMoreElements() ) {
			String name = en.nextElement();
			msg.append( name + "," );
		}
		console.debug( "loggers=" + msg.toString() );
		Logger logger;
		logger = manager.getLogger( "global" );
		printLogger( logger );
		logger = manager.getLogger( "" );
		printLogger( logger );
	}

	private void printLogger( Logger logger ) {
		console.debug( logger.getName() + "=" + logger );
		Handler[] handlers = logger.getHandlers();
		for ( int i = 0; i < handlers.length; i++ ) {
			if ( handlers[i] != null ) {
				console.debug( "\t" + handlers[i] );
			}
		}
	}

	public List<Logger> listLogger() {
		return listLogger;
	}

}
