package	jp.co.sds_corp.util.net;

import java.io.IOException;
import java.net.InetAddress;
import java.net.SocketException;

/**
* IPパケットを送信するソケットの基底クラスです。
*/
public class RawSocket
{
	static Class<?> implClass;
	RawSocketImpl impl;

	static {
		String prop = "jp.co.sds_corp.util.net.impl";
		String cls = "";
		try {
			cls = System.getProperty( prop );
			implClass = Class.forName( cls );
		} catch ( Exception ex ) {
			cls = "jp.co.sds_corp.util.net.PlainRawSocketImpl";
		}

		try {
			implClass = Class.forName( cls );
		} catch ( Exception ex ) {
			throw new Error( "System property " + prop + " incorrect."
							+ " Cannot load " + cls );
		}
	}

	/**
	* ソケットを構築します。
	*/
	public RawSocket() throws SocketException {
		create( 0, null );
	}

	/**
	* ソケットを構築し指定されたアドレスにバインドします。
	* @param	laddr	バインドするアドレス
	*/
	public RawSocket( InetAddress laddr ) throws SocketException {
		create( 0, laddr );
	}

	/**
	* 指定されたプロトコルでソケットを初期化します。
	* @param	protocol	プロトコル
	*/
	public RawSocket( int protocol ) throws SocketException {
		create( protocol, null );
	}

	/**
	* 指定されたプロトコルでソケットを構築し、アドレスにバインドします。
	* @param	protocol	プロトコル
	* @param	laddr	バインドするアドレス
	*/
	public RawSocket( int protocol, InetAddress laddr ) throws SocketException {
		create( protocol, laddr );
	}

	/**
	* 指定されたプロトコルでソケットを初期化し、アドレスにバインドします。
	* @param	protocol	プロトコル
	* @param	laddr	バインドするアドレス
	*/
	void create( int protocol, InetAddress laddr ) throws SocketException {
		try {
			impl = (RawSocketImpl )implClass.newInstance();
		} catch ( Exception ex ) {
			throw new SocketException( "can't instantiate RawSocketImpl" );
		}
		impl.create( protocol );
		if ( laddr != null ) {
			impl.bind( laddr );	
		}
	}

	/**
	* 指定されたIPパケットを送信します。
	* @param	packet	送信するパケット
	*/
	public void send( RawPacket packet ) throws IOException  {
		if ( packet == null ) {
			throw ( new NullPointerException( "packet is null" ));
		}
		SecurityManager security = System.getSecurityManager();

		synchronized ( packet ) {
			if ( security != null ) {
				if ( packet.getAddress().isMulticastAddress() ) {
					security.checkMulticast( packet.getAddress() );
				} else {
					security.checkConnect(
						packet.getAddress().getHostAddress(), -1 );
				}
			}
			impl.send( packet );
		}
	}

	/**
	* ソケットからデータを受信します。
	* @param	packet	受信データを格納するパケット
	*/
	public synchronized void receive( RawPacket packet ) throws IOException {
		if ( packet == null ) {
			throw ( new NullPointerException( "packet is null" ));
		}
		synchronized ( packet ) {
			impl.receive( packet );
		}
	}

	/**
	* ソケットを閉じます。
	*/
	public void close() {
		impl.close();
		impl.fd = null;
	}

	/**
	* ソケットのTTL(Time To Live)を設定します。
	*/
	public void setTTL( int ttl ) throws SocketException {
		impl.setOption( 1, Integer.valueOf( ttl ));
	}
}
